# cody.py (Explorer Workshop)
# Copyright: 2024 Bradan Lane STUDIO
#
# Open Source License: MIT

'''
This CircuitPython program is a sample from the 'Hack the Badge' workshop

Implementation Notes
--------------------
**Hardware:**
    * one of the Explorer Badge designs

**Software and Dependencies:**
    * CircuitPython firmware (UF2): https://circuitpython.org
    * Adafruit CircuitPython Libraries: https://circuitpython.org/libraries

**Open Source Repository:**
    * GitLab: https://gitlab.com/bradanlane/exp_tutorial
    
'''

import time
import board
import digitalio
import displayio
import terminalio       # only used if we use the simple built-in font (and as a fallback if our bitmap font is missing)

display_enable = digitalio.DigitalInOut(board.DISPLAY_EN)
display_enable.direction = digitalio.Direction.OUTPUT
display_enable.value = True # turn on the display power

from adafruit_bitmap_font import bitmap_font
from adafruit_display_text import label

WIDTH:int  = 200
HEIGHT:int = 200
BLACK:int  = 0x000000
WHITE:int  = 0xFFFFFF
RED:int    = 0xFF0000

display = board.DISPLAY

# start building our display groups and tiles
display.root_group = displayio.Group()  # an empty group to start

# start with a red background
background_bitmap = displayio.Bitmap(WIDTH, HEIGHT, 1)
palette = displayio.Palette(1)
palette[0] = WHITE
background_tile = displayio.TileGrid(background_bitmap, pixel_shader=palette)
display.root_group.append(background_tile)

# add test message is various font sizes
#BASE_NAME = "sedgwick"
vid = board.VID()	# get the type of board (which tells us the type of display)
if (vid == 2) or (vid == 4):
    BASE_NAME = "neuropolx"
else:
    BASE_NAME = "graffiti"

font_sizes = [4, 6, 9, 12, 18]

msg = "Fix The Planet"
offset = 0

for size in font_sizes:
    fullname = f"/fonts/{BASE_NAME}{size:d}.bdf"
    try:
        font = bitmap_font.load_font(fullname)
        font_height = font.ascent + font.descent
        box = font.get_bounding_box()
        box_height = box[1] - box[3]
        # use the larger of font_height and box_height
        if font_height < box_height:
            font_height = box_height
        scale = 1
    except:
        print(f"Warning: unable to locate font '{fullname}'")
        print("         will fall back to terminal font")
        font = terminalio.FONT
        font_width, font_height = font.get_bounding_box()
        scale = int(size / 2)
        font_height *= scale
        
    print(msg, "at Y =", offset)
    text = label.Label(font, text=msg, color=RED, scale=scale, anchor_point=(0.0,0.0), anchored_position=(0, offset))
    display.root_group.append(text)
    offset += font_height
    if offset > HEIGHT:
        break

# render everything on the display
wait = display.time_to_refresh
print (f"Refresh will be delayed by {wait:4.1f}.")
time.sleep(wait)

try:
    display.refresh()
    print("ePaper display updated")
except Exception as e:
    print("Error: unable to update display -", e)


counter: int = 0

reported: bool = False

print("Seconds: ", end="")

try:
    while True:
        print(counter, end=" ")
        if not reported and not display.busy:
            print("\nThe ePaper display has finished repainting.")
            reported = True
        time.sleep(1.0)
        counter += 1

finally:
    print("Disabling the ePaper display.")
    display_enable.value = False
    